package vn.teko.loyalty.loyalty_flutter

import vn.teko.loyalty.core.MemberInfoRequestType
import vn.teko.loyalty.core.model.api.query.TransactionHistoryQuery
import vn.teko.loyalty.core.model.api.request.RegisterMemberRequest
import vn.teko.loyalty.core.model.api.response.Pagination
import vn.teko.loyalty.core.model.configuration.LoyaltyConfiguration
import vn.teko.loyalty.core.model.exception.LoyaltyException
import vn.teko.loyalty.core.model.member.MemberInfo
import vn.teko.loyalty.core.model.member.MembershipQr
import vn.teko.loyalty.core.model.member.RegisterMemberResult
import vn.teko.loyalty.core.model.network.NetworkConfig
import vn.teko.loyalty.core.model.network.Tier
import vn.teko.loyalty.core.model.transaction.Transaction
import vn.teko.loyalty.core.model.transaction.TransactionHistoryResult

internal object ModelConverter {
    fun convertLoyaltyConfiguration(config: LoyaltyConfiguration): LoyaltyFlutter.FlutterLoyaltyConfigurationResult {
        val result =
            LoyaltyFlutter.FlutterLoyaltyConfigurationResult().apply { code = ResultCode.success }
        val flutterConfig = LoyaltyFlutter.FlutterLoyaltyConfiguration().apply {
            url = config.url
            bffUrl = config.bffUrl
            merchantCode = config.merchantCode
            clientId = config.clientId
            iamAudience = config.iamAudience
            linkMembershipUrl = config.linkMembershipUrl
            registerMemberUrl = config.registerMemberUrl
            allowLinkMembership = config.allowLinkMembership
            allowRegisterMembership = config.allowRegisterMembership
            membershipCodeFormat = config.membershipCodeFormat
        }

        result.loyaltyConfiguration = flutterConfig
        return result
    }

    fun convertLoyaltyRequest(request: LoyaltyFlutter.FlutterRegisterMemberRequest): RegisterMemberRequest {
        val gender = when (request.gender) {
            LoyaltyFlutter.FlutterGender.female -> RegisterMemberRequest.Gender.FEMALE
            LoyaltyFlutter.FlutterGender.male -> RegisterMemberRequest.Gender.MALE
            LoyaltyFlutter.FlutterGender.none -> RegisterMemberRequest.Gender.NONE
            else -> RegisterMemberRequest.Gender.NONE
        }

        return RegisterMemberRequest(
            phone = request.phone ?: "",
            name = request.name ?: "",
            address = request.address,
            dateOfBirth = request.dateOfBirth,
            email = request.email,
            gender = gender,
            idCardNumber = request.idCardNumber,
            idCitizenNumber = request.idCitizenNumber,
            passportNumber = request.passportNumber,
        )
    }

    fun convertRegisterMemberResult(registerMemberResult: RegisterMemberResult): LoyaltyFlutter.FlutterRegisterMemberResult {
        return LoyaltyFlutter.FlutterRegisterMemberResult().apply {
            code = ResultCode.success
            registerMember = LoyaltyFlutter.FlutterRegisterMember().also { registerMember ->
                registerMember.memberId = registerMemberResult.memberId
            }
        }
    }

    fun convertMembershipQrResult(membershipQr: MembershipQr): LoyaltyFlutter.FlutterMembershipQrResult {
        val membershipQr = LoyaltyFlutter.FlutterMembershipQr().apply {
            qrData = membershipQr.qrData
            issueAt = membershipQr.issueAt
            expireAt = membershipQr.expireAt
        }

        return LoyaltyFlutter.FlutterMembershipQrResult().apply {
            code = ResultCode.success
            this.membershipQr = membershipQr
        }
    }

    fun convertMemberInfoRequest(request: LoyaltyFlutter.FlutterMemberInfoRequest): MemberInfoRequestType {
        if (request.phone.isNullOrBlank() && request.memberQrCode.isNullOrBlank()) {
            return MemberInfoRequestType.CurrentUser
        }

        if (!request.phone.isNullOrBlank()) {
            return MemberInfoRequestType.Phone(request.phone!!)
        }

        return MemberInfoRequestType.MemberQrCode(request.memberQrCode!!)
    }

    fun convertMemberInfoResult(memberInfo: MemberInfo): LoyaltyFlutter.FlutterMemberInfoResult {
        val flutterResult =
            LoyaltyFlutter.FlutterMemberInfoResult().apply { code = ResultCode.success }
        flutterResult.flutterMemberInfo = memberInfo.toFlutterMemberInfo()
        return flutterResult
    }

    fun convertTransactionHistoryRequest(request: LoyaltyFlutter.FlutterTransactionHistoryRequest): TransactionHistoryQuery {
        return TransactionHistoryQuery().apply {
            this.fromTime = request.fromTime
            this.toTime = request.toTime
            this.txnType = request.txnType.toNativeTransactions()
            this.page = request.page?.toInt()
            this.pageSize = request.pageSize?.toInt()
        }
    }

    fun convertTransactionHistoryResult(transactionHistoryResult: TransactionHistoryResult): LoyaltyFlutter.FlutterTransactionHistoryResult {
        val flutterResult =
            LoyaltyFlutter.FlutterTransactionHistoryResult().apply { code = ResultCode.success }

        val flutterTransactions = transactionHistoryResult.transactions.toFlutterTransactions()
        val flutterPagination = transactionHistoryResult.pagination.toFlutterPagination()

        flutterResult.transactionHistory = LoyaltyFlutter.FlutterTransactionHistory().apply {
            this.transactions = flutterTransactions
            this.pagination = flutterPagination
        }

        return flutterResult
    }

    fun convertNetworkConfig(networkConfigResult: NetworkConfig): LoyaltyFlutter.FlutterNetworkConfigResult {
        val flutterResult =
            LoyaltyFlutter.FlutterNetworkConfigResult().apply { code = ResultCode.success }

        val flutterNetworkTiers = networkConfigResult.networkTiers.toFlutterTiers()

        flutterResult.networkConfig = LoyaltyFlutter.FlutterNetworkConfig().also {
            it.networkId = networkConfigResult.networkId
            it.networkName = networkConfigResult.networkName
            it.exchangeRate = networkConfigResult.exchangeRate.toLong()
            it.currencyImg = networkConfigResult.currencyImg
            it.currencyName = networkConfigResult.currencyName
            it.cardDefaultImg = networkConfigResult.cardDefaultImg
            it.avatarDefaultImg = networkConfigResult.avatarDefaultImg
            it.allowToGetMemberByPhone = networkConfigResult.allowToGetMemberByPhone
            it.faqWebUrl = networkConfigResult.faqWebUrl
            it.benefitWebUrl = networkConfigResult.benefitWebUrl
            it.brandColor = networkConfigResult.brandColor
            it.allowMerchantIssueMemberCard = networkConfigResult.allowMerchantIssueMemberCard
            it.networkTiers = flutterNetworkTiers
        }

        return flutterResult
    }

    private fun MemberInfo.toFlutterMemberInfo(): LoyaltyFlutter.FlutterMemberInfo {
        val flutterDefaultCardType = when (defaultCardType) {
            MemberInfo.CardType.MEMBER_CARD -> LoyaltyFlutter.FlutterCardType.memberCard
            else -> LoyaltyFlutter.FlutterCardType.qr
        }

        return LoyaltyFlutter.FlutterMemberInfo().also {
            it.memberQrCode = memberQrCode
            it.memberId = memberId
            it.phone = phone
            it.memberCardId = memberCardId
            it.name = name
            it.isActive = isActive
            it.point = point
            it.tierPoint = tierPoint
            it.keepTierUntil = keepTierUntil
            it.accumulationFrom = accumulationFrom
            it.accumulationTo = accumulationTo
            it.currentTierCode = currentTierCode
            it.currentTierName = currentTierName
            it.currentTierMinPoint = currentTierMinPoint?.toLong()
            it.nextTierName = nextTierName
            it.nextTierMinPoint = nextTierMinPoint?.toLong()
            it.defaultCardType = flutterDefaultCardType
        }

    }

    private fun List<Tier>?.toFlutterTiers(): List<LoyaltyFlutter.FlutterTier>? {
        return this?.map { tier ->
            LoyaltyFlutter.FlutterTier().also {
                it.code = tier.code
                it.name = tier.name
                it.fromPoint = tier.fromPoint.toLong()
                it.cardTieredImg = tier.cardTieredImg
                it.tierIconImg = tier.tierIconImg
            }
        }
    }

    private fun Transaction.TransactionType.toFlutterType(): LoyaltyFlutter.FlutterTransactionType {
        return when (this) {
            Transaction.TransactionType.GRANT -> LoyaltyFlutter.FlutterTransactionType.grant
            Transaction.TransactionType.SPEND -> LoyaltyFlutter.FlutterTransactionType.spend
            Transaction.TransactionType.RETURN_GRANT -> LoyaltyFlutter.FlutterTransactionType.returnGrant
            Transaction.TransactionType.RETURN_SPEND -> LoyaltyFlutter.FlutterTransactionType.returnSpend
            else -> LoyaltyFlutter.FlutterTransactionType.grant
        }
    }

    private fun Transaction.Status.toFlutterStatus(): LoyaltyFlutter.FlutterStatus {
        return when (this) {
            Transaction.Status.INIT -> LoyaltyFlutter.FlutterStatus.init
            Transaction.Status.INIT_FAILURE -> LoyaltyFlutter.FlutterStatus.initFailure
            Transaction.Status.NO_ACCOUNTING -> LoyaltyFlutter.FlutterStatus.noAccounting
            Transaction.Status.ACCOUNTING_SUCCESS -> LoyaltyFlutter.FlutterStatus.accountingSuccess
            Transaction.Status.ACCOUNTING_FAILURE -> LoyaltyFlutter.FlutterStatus.accountingFailure
            Transaction.Status.ACCOUNTING -> LoyaltyFlutter.FlutterStatus.accounting
            else -> LoyaltyFlutter.FlutterStatus.accountingSuccess
        }
    }

    private fun LoyaltyFlutter.FlutterTransactionRequestType?.toNativeTransactions(): TransactionHistoryQuery.QueryTransactionType? {
        return when (this) {
            LoyaltyFlutter.FlutterTransactionRequestType.all -> TransactionHistoryQuery.QueryTransactionType.ALL
            LoyaltyFlutter.FlutterTransactionRequestType.grant -> TransactionHistoryQuery.QueryTransactionType.GRANT
            LoyaltyFlutter.FlutterTransactionRequestType.spend -> TransactionHistoryQuery.QueryTransactionType.SPEND
            else -> null
        }
    }

    private fun Pagination.toFlutterPagination(): LoyaltyFlutter.FlutterPagination {
        return LoyaltyFlutter.FlutterPagination().also { flutterPagination ->
            flutterPagination.page = page.toLong()
            flutterPagination.pageSize = pageSize.toLong()
            flutterPagination.totalItems = totalItems.toLong()
        }
    }

    private fun List<Transaction>.toFlutterTransactions(): List<LoyaltyFlutter.FlutterTransaction> {
        return map { transaction ->
            val accountingType = when (transaction.accountingType) {
                Transaction.AccountingType.CREDIT -> LoyaltyFlutter.FlutterAccountingType.credit
                else -> LoyaltyFlutter.FlutterAccountingType.debit
            }

            LoyaltyFlutter.FlutterTransaction().apply {
                this.id = transaction.id
                this.refId = transaction.refId
                this.orderCode = transaction.orderCode
                this.txnType = transaction.txnType.toFlutterType()
                this.txnTypeText = transaction.txnTypeText
                this.txnDesc = transaction.txnDesc
                this.accountingType = accountingType
                this.status = transaction.status.toFlutterStatus()
                this.point = transaction.point.toLong()
                this.createdAt = transaction.createdAt
                this.networkId = transaction.networkId
                this.networkName = transaction.networkName
                this.merchantCode = transaction.merchantCode
                this.merchantName = transaction.merchantName
            }
        }
    }
}