//
//  ModelConverter.swift
//  loyalty_flutter
//
//  Created by Cao Trung on 22/06/2022.
//

import Foundation
import LoyaltyCore
import LoyaltyModel

func convertLoyaltyConfiguration(config: LoyaltyServiceConfig) -> FlutterLoyaltyConfigurationResult {
    let result = FlutterLoyaltyConfigurationResult()
    result.code = success
    
    let flutterConfig = FlutterLoyaltyConfiguration()
    flutterConfig.linkMembershipUrl = config.linkMembershipUrl
    flutterConfig.registerMemberUrl = config.registerMemberUrl
    flutterConfig.allowLinkMembership = convertBoolToNSNumber(config.allowLinkMembership)
    flutterConfig.allowRegisterMembership = convertBoolToNSNumber(config.allowRegisterMembership)
    flutterConfig.membershipCodeFormat = config.membershipCodeFormat.rawValue
    
    result.loyaltyConfiguration = flutterConfig
    
    return result
}

func convertLoyaltyRegisterMemberRequest(request: FlutterRegisterMemberRequest) -> RegisterMemberRequestParam {
    return RegisterMemberRequestParam(
        name: request.name ?? "",
        phone: request.phone ?? "",
        email: request.email,
        address: request.address,
        dateOfBirth: request.dateOfBirth,
        gender: convertGender(flutterGender:  request.gender),
        idCardNumber: request.idCardNumber,
        idCitizenNumber: request.idCitizenNumber,
        passportNumber: request.passportNumber
    )
}

private func convertGender(flutterGender: FlutterGender) -> RegisterMemberRequestParam.Gender? {
    switch flutterGender {
    case FlutterGender.female:
        return RegisterMemberRequestParam.Gender.female
    case FlutterGender.male:
        return RegisterMemberRequestParam.Gender.male
    case FlutterGender.none:
        return RegisterMemberRequestParam.Gender.undefined
    default:
        return nil
    }
}

func convertRegisterMemberResult(registerMemberResult: RegisterMemberResult) -> FlutterRegisterMemberResult {
    let result = FlutterRegisterMemberResult()
    let registerMember = FlutterRegisterMember()
    registerMember.memberId = registerMemberResult.memberId
    
    result.code = success
    result.registerMember = registerMember
    
    return result
}

func convertMembershipQrResult(encryptedMemCode: QRData) -> FlutterMembershipQrResult {
    let membershipQr = FlutterMembershipQr()
    membershipQr.qrData = encryptedMemCode.code
    membershipQr.issueAt = NSNumber(value: encryptedMemCode.issueAt)
    membershipQr.expireAt = NSNumber(value: encryptedMemCode.expireAt)
    
    let flutterResult =  FlutterMembershipQrResult()
    flutterResult.code = success
    flutterResult.membershipQr = membershipQr
    
    return flutterResult
}

func convertMemberInfoType(request: FlutterMemberInfoRequest) -> MemberInfoRequestType {
    if (request.phone == nil && request.memberQrCode == nil) {
        return MemberInfoRequestType.currentUser
    }
    
    if (request.phone != nil) {
        return MemberInfoRequestType.phone(request.phone!)
    }
    
    return MemberInfoRequestType.memberQrCode(request.memberQrCode!)
}

func convertMemberInfoResult(memberInfo: MemberInfo) -> FlutterMemberInfoResult {
    let flutterResult = FlutterMemberInfoResult()
    flutterResult.code  = success
    
    let flutterMemberInfo = FlutterMemberInfo()
    flutterMemberInfo.memberQrCode = memberInfo.memberQrCode
    flutterMemberInfo.memberId = memberInfo.memberId
    flutterMemberInfo.phone = memberInfo.phone
    flutterMemberInfo.memberCardId = memberInfo.memberCardId
    flutterMemberInfo.name = memberInfo.name
    flutterMemberInfo.isActive = convertBoolToNSNumber(memberInfo.isActive)
    flutterMemberInfo.point = NSNumber(value: memberInfo.point)
    flutterMemberInfo.tierPoint = convertIntToNSNumber(memberInfo.tierPoint)
    flutterMemberInfo.keepTierUntil = convertIntToNSNumber(memberInfo.keepTierUntil)
    flutterMemberInfo.accumulationFrom = convertIntToNSNumber(memberInfo.accumulationFrom)
    flutterMemberInfo.accumulationTo = convertIntToNSNumber(memberInfo.accumulationTo)
    flutterMemberInfo.currentTierCode = memberInfo.currentTierCode
    flutterMemberInfo.currentTierName = memberInfo.currentTierName
    flutterMemberInfo.currentTierMinPoint = convertIntToNSNumber(memberInfo.currentTierMinPoint)
    flutterMemberInfo.nextTierName = memberInfo.nextTierName
    flutterMemberInfo.nextTierMinPoint = convertIntToNSNumber(memberInfo.nextTierMinPoint)
    flutterMemberInfo.defaultCardType = convertFlutterCardType(card: memberInfo.defaultCardType ?? LoyaltyCardType.qr)
    
    flutterResult.flutterMemberInfo = flutterMemberInfo
    return flutterResult
}

func convertTransactionHistoryRequeset(request: FlutterTransactionHistoryRequest) -> TransactionHistoryRequestParams {
    
    let param = TransactionHistoryRequestParams(
        txnType: request.txnType.toTxnType(),
        fromTime: request.fromTime,
        toTime: request.toTime,
        page: request.page?.intValue ?? 1,
        size: request.pageSize?.intValue ?? 20
    )
    
    return param
}

func convertTransactionHistoryResult(historyResponse: TransactionHistoryResponse) -> FlutterTransactionHistoryResult {
    let flutterResult = FlutterTransactionHistoryResult()
    guard let transactionHistory = historyResponse.data else {
        flutterResult.code = unexpected
        flutterResult.error = "Missing Data Error"
        return flutterResult
    }
    
    let transactions = transactionHistory.transactions.toFlutterTransactions()
    let pagination = transactionHistory.pagination.toFlutterPaging()
    
    let flutterTransactionHistory = FlutterTransactionHistory()
    flutterTransactionHistory.transactions = transactions
    flutterTransactionHistory.pagination = pagination
    
    flutterResult.transactionHistory = flutterTransactionHistory
    flutterResult.code = success
    return flutterResult
}

private func convertAccoutingType(accounting: AccountingType) -> FlutterAccountingType {
    switch accounting {
    case AccountingType.credit:
        return FlutterAccountingType.credit
    case AccountingType.debit:
        return FlutterAccountingType.debit
    default:
        return FlutterAccountingType.debit
    }
}

func convertNetworkConfig(networkConfigResponse: NetworkConfigResponse) -> FlutterNetworkConfigResult {
    let flutterResult = FlutterNetworkConfigResult()
    
    guard let networkConfigResult = networkConfigResponse.config else {
        flutterResult.code = unexpected
        flutterResult.error = "Missing Data Error"
        return flutterResult
    }
    
    flutterResult.code = success
    
    let flutterNetworkTiers = networkConfigResult.networkTiers?.map { tier -> FlutterTier in
        let flutterTier = FlutterTier()
        flutterTier.code = tier.code
        flutterTier.name = tier.name
        flutterTier.fromPoint = convertIntToNSNumber(tier.fromPoint)
        flutterTier.cardTieredImg = tier.cardTieredImg
        flutterTier.tierIconImg = tier.cardTieredImg
        
        return flutterTier
    }
    
    let flutterNetworkData = FlutterNetworkConfig()
    flutterNetworkData.networkId = String(networkConfigResult.networkId)
    flutterNetworkData.networkName = networkConfigResult.networkName
    flutterNetworkData.exchangeRate = convertIntToNSNumber(networkConfigResult.exchangeRate)
    flutterNetworkData.currencyImg = networkConfigResult.currencyImg
    flutterNetworkData.currencyName = networkConfigResult.currencyName
    flutterNetworkData.cardDefaultImg = networkConfigResult.cardDefaultImg
    flutterNetworkData.avatarDefaultImg = networkConfigResult.avatarDefaultImg
    flutterNetworkData.allowToGetMemberByPhone = convertBoolToNSNumber(networkConfigResult.allowToGetMemberByPhone)
    flutterNetworkData.faqWebUrl = networkConfigResult.faqWebUrl
    flutterNetworkData.benefitWebUrl = networkConfigResult.benefitWebUrl
    flutterNetworkData.brandColor = networkConfigResult.brandColor
    flutterNetworkData.allowMerchantIssueMemberCard = convertBoolToNSNumber(networkConfigResult.allowMerchantIssueMemberCard)
    flutterNetworkData.networkTiers = flutterNetworkTiers
    
    flutterResult.networkConfig = flutterNetworkData
    return  flutterResult
}

private func convertIntToNSNumber(_ value: Int?) -> NSNumber? {
    guard let value = value else {
        return nil
    }
    
    return NSNumber(value: value)
}

private func convertBoolToNSNumber(_ value: Bool?) -> NSNumber? {
    guard let value = value else {
        return nil
    }
    
    return NSNumber(value: value)
}

extension Array where Element == Transaction {
    func toFlutterTransactions() -> [FlutterTransaction] {
        let flutterTransactions = map { transaction -> FlutterTransaction in
            let flutterTransaction = FlutterTransaction()
            flutterTransaction.id = transaction.id
            flutterTransaction.refId = transaction.refId
            flutterTransaction.orderCode = transaction.orderCode
            flutterTransaction.txnType = convertTransactionType(type: transaction.txnType)
            flutterTransaction.txnTypeText = transaction.txnTypeText
            flutterTransaction.txnDesc = transaction.txnDesc
            flutterTransaction.accountingType = convertAccoutingType(accounting: transaction.accountingType)
            flutterTransaction.status = convertStatus(status: transaction.status)
            flutterTransaction.point = NSNumber(value: transaction.point)
            flutterTransaction.createdAt = NSNumber(value: transaction.createdAt)
            flutterTransaction.networkId = NSNumber(value: transaction.networkId)
            flutterTransaction.networkName = transaction.networkName
            flutterTransaction.merchantCode = transaction.merchantCode
            flutterTransaction.merchantName = transaction.merchantName
            
            return flutterTransaction
            
        }
        
        return flutterTransactions
    }
}

private func convertStatus(status: TransactionStatus) -> FlutterStatus {
    switch status {
    case TransactionStatus.`init`:
        return FlutterStatus.`init`
    case TransactionStatus.failInit:
        return FlutterStatus.initFailure
    case TransactionStatus.unfinished:
        return FlutterStatus.noAccounting
    case TransactionStatus.finished:
        return FlutterStatus.accountingSuccess
    case TransactionStatus.fail:
        return FlutterStatus.accountingFailure
    case TransactionStatus.processing:
        return FlutterStatus.accounting
    default:
        return FlutterStatus.`init`
    }
}


private func convertTransactionType(type: TransactionType) -> FlutterTransactionType {
    switch type {
    case TransactionType.grant:
        return FlutterTransactionType.grant
    case TransactionType.spend:
        return FlutterTransactionType.spend
    case TransactionType.returnGrant:
        return FlutterTransactionType.returnGrant
    case TransactionType.returnSpend:
        return FlutterTransactionType.returnSpend
    default:
        return FlutterTransactionType.grant
    }
}

private func convertFlutterCardType(card: LoyaltyCardType) -> FlutterCardType {
    switch card {
    case LoyaltyCardType.memberCard:
        return FlutterCardType.memberCard
    case LoyaltyCardType.qr:
        return FlutterCardType.qr
    }
}

extension NetworkConfig {
    func toFlutterNetworkConfig() -> FlutterNetworkConfig {
        let flutterNetworkConfig = FlutterNetworkConfig()
        flutterNetworkConfig.exchangeRate = convertIntToNSNumber(exchangeRate)
        flutterNetworkConfig.currencyImg = currencyImg
        flutterNetworkConfig.currencyName = currencyName
        flutterNetworkConfig.cardDefaultImg = cardDefaultImg
        flutterNetworkConfig.avatarDefaultImg = avatarDefaultImg
        flutterNetworkConfig.allowToGetMemberByPhone = convertBoolToNSNumber(allowToGetMemberByPhone)
        flutterNetworkConfig.faqWebUrl = faqWebUrl
        flutterNetworkConfig.benefitWebUrl = benefitWebUrl
        flutterNetworkConfig.brandColor = brandColor
        flutterNetworkConfig.allowMerchantIssueMemberCard = convertBoolToNSNumber(allowMerchantIssueMemberCard)
        
        return flutterNetworkConfig
    }
}

extension Optional where Wrapped == LoyaltyPaging {
    func toFlutterPaging() -> FlutterPagination {
        let pagination = FlutterPagination()
        
        pagination.page = convertIntToNSNumber(self?.currentPage)
        pagination.pageSize = convertIntToNSNumber(self?.pageSize)
        pagination.totalItems = convertIntToNSNumber(self?.totalItems)
        
        return pagination
    }
}

extension FlutterTransactionRequestType {
    func toTxnType() -> TransactionHistoryRequestParams.TxnType {
        switch self {
        case FlutterTransactionRequestType.grant:
            return TransactionHistoryRequestParams.TxnType.grant
        case FlutterTransactionRequestType.spend:
            return TransactionHistoryRequestParams.TxnType.spend
        case FlutterTransactionRequestType.all:
            return TransactionHistoryRequestParams.TxnType.all
        default:
            return TransactionHistoryRequestParams.TxnType.all
        }
    }
}


extension LoyaltyError {
    func toFlutterCommonResultError() -> CommonResult {
        let flutterResult = CommonResult()
        let errorData = convertCodeToException(loyaltyError: self)
        flutterResult.code = errorData.code
        flutterResult.error = errorData.message
        
        return flutterResult
    }
    
    func toFlutterLoyaltyConfigurationResultError() -> FlutterLoyaltyConfigurationResult {
        let flutterResult = FlutterLoyaltyConfigurationResult()
        let errorData = convertCodeToException(loyaltyError: self)
        flutterResult.code = errorData.code
        flutterResult.error = errorData.message
        
        return flutterResult
    }
    
    func toFlutterRegisterMemberResultError() -> FlutterRegisterMemberResult {
        let flutterResult = FlutterRegisterMemberResult()
        let errorData = convertCodeToException(loyaltyError: self)
        flutterResult.code = errorData.code
        flutterResult.error = errorData.message
        
        return flutterResult
    }
    
    func toFlutterMembershipQrResultError() -> FlutterMembershipQrResult {
        let flutterResult = FlutterMembershipQrResult()
        let errorData = convertCodeToException(loyaltyError: self)
        flutterResult.code = errorData.code
        flutterResult.error = errorData.message
        
        return flutterResult
    }
    
    func toFlutterMemberInfoResultError() -> FlutterMemberInfoResult {
        let flutterResult = FlutterMemberInfoResult()
        let errorData = convertCodeToException(loyaltyError: self)
        flutterResult.code = errorData.code
        flutterResult.error = errorData.message
        
        return flutterResult
    }
    
    func toFlutterTransactionHistoryResultError() -> FlutterTransactionHistoryResult {
        let flutterResult = FlutterTransactionHistoryResult()
        let errorData = convertCodeToException(loyaltyError: self)
        flutterResult.code = errorData.code
        flutterResult.error = errorData.message
        
        return flutterResult
    }
    
    
    func toFlutterNetworkConfigResultError() -> FlutterNetworkConfigResult {
        let flutterResult = FlutterNetworkConfigResult()
        let errorData = convertCodeToException(loyaltyError: self)
        flutterResult.code = errorData.code
        flutterResult.error = errorData.message
        
        return flutterResult
    }
    
    func toFlutterClientIdResultError() -> FlutterClientIdResult {
        let flutterResult = FlutterClientIdResult()
        let errorData = convertCodeToException(loyaltyError: self)
        flutterResult.code = errorData.code
        flutterResult.error = errorData.message
        
        return flutterResult
    }
}