import 'exception/loyalty_core_exception.dart';
import 'model/loyalty_configuration.dart';
import 'model/member_info.dart';
import 'model/member_info_request.dart';
import 'model/membership_qr.dart';
import 'model/netword_config_result.dart';
import 'model/register_member_request.dart';
import 'model/register_member_result.dart';
import 'model/transaction_history.dart';
import 'model/transaction_history_request.dart';
import 'result/result.dart';
import 'src/loyalty_api.dart' as pigeon;
import 'src/model_converter.dart';
import 'src/result_mapper.dart';

class Terraloyalty {
  static final _flutterLoyaltyApi = pigeon.FlutterLoyaltyApi();
  static final Map<String, Terraloyalty> _instances = {};

  final String _appName;

  Terraloyalty._(this._appName);

  static Future<Terraloyalty> getInstance(String appName) async {
    if (_instances[appName] == null) {
      _instances[appName] = await _createNewInstance(appName);
    }

    return _instances[appName]!;
  }

  static Future<Terraloyalty> _createNewInstance(String appName) async {
    final newAuthInstance = Terraloyalty._(appName);
    try {
      await _flutterLoyaltyApi.getInstance(appName);
    } on Exception {
      rethrow;
    }

    return newAuthInstance;
  }

  Future<Result<LoyaltyConfiguration, Exception>> getConfiguration() async {
    final result = await _flutterLoyaltyApi.getConfiguration(_appName);

    if (result.isFailed) {
      return Result.failure(result.getConvertedException());
    }

    final config = result.loyaltyConfiguration;
    if (config == null) {
      return Result.failure(MissingDataException());
    }

    return Result.success(ModelConverter.toModelLoyaltyConfiguration(config));
  }

  Future<Result<RegisterMemberResult, Exception>> registerMember(RegisterMemberRequest request) async {
    final result = await _flutterLoyaltyApi.registerMember(
      _appName,
      ModelConverter.toFlutterRegisterMemberRequest(request),
    );

    if (result.isFailed) {
      return Result.failure(result.getConvertedException());
    }

    final registerMember = result.registerMember;
    if (registerMember == null) {
      return Result.failure(MissingDataException());
    }

    return Result.success(RegisterMemberResult(registerMember.memberId ?? ""));
  }

  Future<Result<EncryptedMemCode, Exception>> getMembershipQr() async {
    final result = await _flutterLoyaltyApi.getMembershipQr(_appName);

    if (result.isFailed) {
      return Result.failure(result.getConvertedException());
    }

    final membershipQr = result.membershipQr;
    if (membershipQr == null) {
      return Result.failure(MissingDataException());
    }

    return Result.success(ModelConverter.toModelEncryptedMemCode(membershipQr));
  }

  Future<Result<MemberInfo, Exception>> getMemberInfo(MemberInfoRequest request) async {
    final result = await _flutterLoyaltyApi.getMemberInfo(
        _appName,
        pigeon.FlutterMemberInfoRequest(
          phone: request.phone,
          memberQrCode: request.memberQrCode,
        ));

    if (result.isFailed) {
      return Result.failure(result.getConvertedException());
    }

    final flutterMemberInfo = result.flutterMemberInfo;
    if (flutterMemberInfo == null) {
      return Result.failure(MissingDataException());
    }

    return Result.success(ModelConverter.toModelMemberInfo(flutterMemberInfo));
  }

  Future<Result<TransactionHistory, Exception>> getTransactionHistory(TransactionHistoryRequest request) async {
    final result = await _flutterLoyaltyApi.getTransactionHistory(
        _appName, ModelConverter.toFlutterTransactionHistoryRequest(request));

    if (result.isFailed) {
      return Result.failure(result.getConvertedException());
    }
    final transactionHistory = result.transactionHistory;

    if (transactionHistory == null) {
      return Result.failure(MissingDataException());
    }

    return Result.success(ModelConverter.toModelTransactionHistory(transactionHistory));
  }

  Future<Result<NetworkConfig, Exception>> getNetworkConfig() async {
    final result = await _flutterLoyaltyApi.getNetworkConfig(_appName);

    if (result.isFailed) {
      return Result.failure(result.getConvertedException());
    }
    final networkConfig = result.networkConfig;

    if (networkConfig == null) {
      return Result.failure(MissingDataException());
    }

    return Result.success(ModelConverter.toModelNetworkConfigResult(networkConfig));
  }

  Future<Result<String, Exception>> getClientId() async {
    final result = await _flutterLoyaltyApi.getClientId(_appName);

    if (result.isFailed) {
      return Result.failure(Exception(result.getConvertedException()));
    }

    final clientId = result.clientId;
    if (clientId == null) {
      return Result.failure(MissingDataException());
    }

    return Result.success(clientId);
  }

  Future<Result<void, Exception>> assignMemberCardId(String memberId, String memberCardId) async {
    final result = await _flutterLoyaltyApi.assignMemberCardId(_appName, memberId, memberCardId);

    if (result.isFailed) {
      return Result.failure(Exception(result.getConvertedException()));
    }

    return Result.success({});
  }
}


